/*
 * HTTPARSER.H
 * Contains templates and parser fuctions.
 *
 * Copyright (C) 2008  Acri Emanuele <crossbower@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __HTTPARSER_H__
#define __HTTPARSER_H__

#include <regex.h>
#include <getopt.h>

#include "httsquash.h"

#define VERSION "v0.4"
#define AUTHOR "Acri Emanuele (crossbower@gmail.com)"

/*
 * Regular expression used by
 * parseopt() and parseipr() functions
 */
char *reg_netblock =
    "^[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}[-][0-9]{1,3}$";

char *reg_netsingle =
    "^[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}$";

char *reg_number =
    "\\d+?";


/*
 * Http 1.1 request
 */
char http_request[MAX_DATA] =
"GET / HTTP/1.1\r\n" \
"Host: localhost\r\n" \
"User-Agent: Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.3)\r\n" \
"Accept: text/html\r\n" \
"Connection: close\r\n" \
"\r\n";

/*
 * Program usage
 */
void usage(FILE *out, char *error)
{
    char usage[] =
        "HTTSquash scanner " VERSION " - " AUTHOR "\n" \
        "Usage:\n" \
        "  httsquash -r range [options]\n" \
        "Options:\n" \
        "  -r\tip-range or single ip (example 192.168.1.1-254 or 192.168.1.1)\n" \
        "  -p\tport (default 80)\n" \
        "  -t\ttime in seconds (default 3)\n" \
        "  -P\tpersonalized http request. Payload file\n" \
        "  -v\tfull answer (include html data)\n" \
        "  -j\tcookie jar separator (\"%%\")\n";

    fputs(usage, out);

    if(error)
        fputs(error, out);

    exit(1);
}

/*
 * Regular expression matching function
 * Returns:
 *    -1  error on pattern compilation
 *     0  don't match
 *     1  match
 */
int regmatch(const char* str, char* regex) {
    regex_t re;

    // Compile the regex pattern.
    if (regcomp(&re, regex, REG_EXTENDED | REG_NOSUB) != 0) {
        fprintf(stderr, "Error on regex pattern compilation.\n");
        return -1;
    }

    // Use the pattern on the passed string
    if (regexec(&re, str, 0, NULL, 0))  {
        return 0;
    }

    regfree(&re);
    return 1;
}

/*
 * Read http request from a file
 */
int readhttprequest(char *path) {
    FILE *file;
    char buffer[1024];

    memset(http_request, 0, MAX_DATA);

    if((file=fopen(path, "r"))==NULL)
        usage(stderr, "\nError opening file for read the http request.\n");

    if(!fread(http_request, 1, MAX_DATA, file))
        usage(stderr, "\nError reading http request from file.\n");

    fclose(file);
    
    return 1;
}

/*
 * Fill ip range struct and http_request with arguments
 * received from parseopt();
 * (perhaps you could find a more elegant way...)
 */
int parseipr(struct iprange *ipr, char *range, int port, int time, char *request, int verbose, int cookiejar)
{
    char *c;
    int i=0;

    //iprange.range
    strncpy(ipr->range, range, 22);

    //iprange.netblock
    while(range[i]!='\0') i++;
    while(range[i]!='.') i--;
    c=&range[i];

    *c = '\0';
    strncpy(ipr->netblock, range, 20);
    *c = '.';

    //iprange.addr
    ipr->addr[0] = '\0';

    //iprange.start
    c = strtok(range, ".");
    for(i=0; i<3; i++) c=strtok(NULL, ".");

    ipr->start=atoi(c);

    //iprange.end (single ip or range?)
    
    // - single ip
    if(regmatch(ipr->range, reg_netsingle)) {
        strcat(ipr->range,"-");
        strcat(ipr->range,c);
        ipr->end=ipr->start;
    }
    
    // - range
    else {
        i=0;
        while(range[i]!='-') i++;
        c = &range[i+1];
        
        ipr->end=atoi(c);
    }

    //iprange.curr
    ipr->curr = ipr->start;

    // - range is coerent?
    if(ipr->end < ipr->start)
        usage(stderr, "\nArgument ip-range is impossible ;)\n");
        
    //iprange.port
    ipr->port=port;
    
    //iprange.time
    ipr->time=time;    
    
    //iprange.verbose
    ipr->verbose=verbose;
    
    //iprange.cookiejar
    ipr->cookiejar=cookiejar;
    
    //http_request
    if(request)
        readhttprequest(request);

    return 1;
}

/*
 * Parser for command line options
 * See getopt(3)...
 */
int parseopt(struct iprange *ipr, int argc, char **argv)
{
    char c;
    char *range=NULL, *request=NULL;
    int port=80, time=3, verbose=0, cookiejar=0;
    
    char *shortopt = "r:p:t:P:vj"; // short options
    
    while ((c = getopt (argc, argv, shortopt)) != -1)
        switch (c) {
        
            case 'r': // IP range
                if(!(regmatch(optarg, reg_netblock) || regmatch(optarg, reg_netsingle)))
                    usage(stderr, "\nArgument ip-range is invalid.\n");
                range = optarg;
                break;
            
            case 'p': // Port
                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument port is invalid.\n");
                port = atoi(optarg);
                break;
            
            case 't': // Time
                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument time is invalid.\n");
                time = atoi(optarg);
                break;
            
            case 'P': // Payload file
                request = optarg;
                break;
                
            case 'v': // Full answer
                verbose = 1;
                break;
            
            case 'j': // Cookie jar
                cookiejar = 1;
                break;
                            
            case '?':
            default:
                usage(stderr, NULL);
        }

    //check collected args
    if(range==NULL)
        usage(stderr, NULL);
    if(time<1)
        time = 3;
    if(port<1)
        port=80;

    //fill iprange struct
    parseipr(ipr, range, port, time, request, verbose, cookiejar);
    
    return 1;
}

#endif /* __HTTPARSER_H__ */

