/*
 * LETDOWN.H
 * Usage, parser and packet injection functions...
 *
 * Copyright (C) 2008  Acri Emanuele <crossbower@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LETDOWN_H__
#define __LETDOWN_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <getopt.h>
#include <regex.h>

#define VERSION "v0.4"
#define AUTHOR "Acri Emanuele (crossbower@gmail.com)"

#define MIN_ARG 64
#define MAX_ARG 1024

#define IPTABLES_ENABLE  "iptables -A OUTPUT -p tcp --tcp-flags ALL RST -j DROP"
#define IPTABLES_DISABLE "iptables -D OUTPUT -p tcp --tcp-flags ALL RST -j DROP"

#define BLACKHOLE_ENABLE  "sysctl net.inet.tcp.blackhole=2"
#define BLACKHOLE_DISABLE "sysctl net.inet.tcp.blackhole=0"

/*
 * Options structure
 */
struct options {
    int dport; // destination port
    int xport; // first source port
    int yport; // last source port
    int time;  // sleep time
    int alarm; // alarm time
    
    char dstaddr[MIN_ARG]; // destination address
    char srcaddr[MIN_ARG]; // source address
    char device[MIN_ARG];  // network interface
    
    char firewall[2][MAX_ARG]; // firewall rules
    
    char payload[MAX_ARG*4]; // payload
};

/*
 * Regular expression used by
 * parseopt() function
 */
char *reg_ip_addr =
    "^[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}$";

char *reg_number =
    "\\d+?";
    
/*
 * Program usage
 */
void usage(FILE *out, char *error)
{
    char usage[] =
        "LetDown 3wh+payload flooder " VERSION " - " AUTHOR "\n" \
        "Usage:\n" \
        "  letdown -d destination ip -p port [options]\n" \
        "Options:\n" \
        "  -d\tdestination ip address, target\n" \
        "  -p\tdestination port\n" \
        "  -s\tsource ip address\n" \
        "  -x\tfirst source port (default 1025)\n" \
        "  -y\tlast source port (default 65535)\n" \
        "  -i\tnetwork interface\n" \
        "  -t\tsleep time in microseconds (default 0)\n" \
        "  -a\tmax time in second for waiting responses\n" \
        "  -f\tautomagically set firewall rules for blocking\n" \
        "    \trst packet generated by the kernel\n"
        "    \texamples: -f iptables, -f blackhole (for freebsd)\n" \
        "  -P\tpayload file (see payloads directory...)\n";

    fputs(usage, out);

    if(error)
        fputs(error, out);

    exit(1);
}

/*
 * Regular expression matching function
 * Returns:
 *    -1  error on pattern compilation
 *     0  don't match
 *     1  match
 */
int regmatch(const char* str, char* regex)
{
    regex_t re;

    // Compile the regex pattern.
    if (regcomp(&re, regex, REG_EXTENDED | REG_NOSUB) != 0) {
        fprintf(stderr, "Error on regex pattern compilation.\n");
        return -1;
    }

    // Use the pattern on the passed string
    if (regexec(&re, str, 0, NULL, 0))  {
        return 0;
    }

    regfree(&re);
    return 1;
}

/*
 * Get payload from a file
 */
int getpayload(struct options *opt, char *path) {
    FILE *file;
    char buffer[1024];
    int size=0, i=0;

    memset(opt->payload, 0, MAX_ARG*4);

    if((file=fopen(path, "r"))==NULL)
        usage(stderr, "\nError opening payload file.\n");

    if(!fread(opt->payload, 1, MAX_ARG*4, file))
        usage(stderr, "\nError reading  payload from file.\n");

    fclose(file);
    
    return 1;
}

/*
 * Parser for command line options
 * See getopt(3)...
 */
int parseopt(struct options *opt, int argc, char **argv)
{
    char c;
    
    int  dport = 0,     // destination port
         xport = 1025,  // first source port
         yport = 65535, // last source port
         time  = 0,     // sleep time
         alarm = 0;     // alarm time
    
    char *dstaddr  = NULL, // destination address
         *srcaddr  = NULL, // source address
         *device   = NULL, // network interface
         *firewall = NULL, // firewall rule
         *payload  = NULL; // payload file
    
    memset(opt, 0, sizeof(struct options)); // clear opt
         
    char *shortopt = "d:p:x:y:s:i:t:a:f:P:"; // short options
    
    while ((c = getopt (argc, argv, shortopt)) != -1)
        switch (c) {
        
            case 'd': // Destination
                if(!regmatch(optarg, reg_ip_addr))
                    usage(stderr, "\nArgument destination address is invalid.\n");
                dstaddr = optarg;
                break;
            
            case 'p': // Port
                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument destination port is invalid.\n");
                dport = atoi(optarg);
                break;
            
            case 'x': // First source port
                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument first source port is invalid.\n");
                xport = atoi(optarg);
                break;
            
            case 'y': // Last source port
                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument last source port is invalid.\n");
                yport = atoi(optarg);
                break;
                
            case 's': // Source address
                if(!regmatch(optarg, reg_ip_addr))
                    usage(stderr, "\nArgument source address is invalid.\n");
                srcaddr = optarg;
                break;
            
            case 'i': // Interface
                device = optarg;
                break;
            
            case 't': // Sleep time
                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument time is invalid.\n");
                time = atoi(optarg);
                break;
            
            case 'a': // Alarm time
                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument alarm is invalid.\n");
                alarm = atoi(optarg);
                break;
            
            case 'f': // Firewall rule
                firewall = optarg;
                break;
            
            case 'P': // Payload
                payload = optarg;
                break;
                            
            case '?':
            default:
                usage(stderr, NULL);
        }

    //check collected args and fill options struct
    if(!dstaddr || dport<1)
        usage(stderr, NULL);
    else {
        strncpy(opt->dstaddr, dstaddr, MIN_ARG);
        opt->dport=dport;
    }
    
    if(srcaddr)
        strncpy(opt->srcaddr, srcaddr, MIN_ARG);
    
    if(time<0)
        time = 0;
    if(alarm<0)
        alarm = 0;
    if(xport>yport)
        usage(stderr, "\nAre you ok? Check -x and -y please ;)\n");
    
    if(firewall) {
        if(!strcmp("iptables", firewall)) {
            strncpy(opt->firewall[0], IPTABLES_ENABLE, MAX_ARG);
            strncpy(opt->firewall[1], IPTABLES_DISABLE, MAX_ARG);
        }
        else if(!strcmp("blackhole", firewall)) {
            strncpy(opt->firewall[0], BLACKHOLE_ENABLE, MAX_ARG);
            strncpy(opt->firewall[1], BLACKHOLE_DISABLE, MAX_ARG);
        }
        else
           usage(stderr, "\nI don't know this firewall\n");
    }

    opt->xport=xport;
    opt->yport=yport;
    opt->time=time;
    opt->alarm=alarm;
    
    if(device)
        strncpy(opt->device, device, MIN_ARG);
    
    if(payload)
        getpayload(opt, payload);
    
    return 1;
}

#endif /* __LETDOWN_H__ */

