/*
 * REVERSERAIDER.H
 * Usage and options parser...
 *
 * Copyright (C) 2008  Acri Emanuele <crossbower@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __REVERSERAIDER_H__
#define __REVERSERAIDER_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <getopt.h>
#include <regex.h>

#define VERSION "v0.4"
#define AUTHOR "Acri Emanuele (crossbower@gmail.com)"

#define MIN_ARG 64
#define MAX_ARG 1024

/*
 * Options structure
 */
struct options {
    char domain[MAX_ARG];   // domain
    char range[MIN_ARG];    // range
    char wordlist[MAX_ARG]; // wordlist
    int time;               // time
    int permutation;        // permutation
    
    int start;              // for reverse scanning
    int end;                // ...
    char block[MIN_ARG];    // ...
};

/*
 * Regular expression used by
 * parseopt() function
 */
char *reg_ip_range =
    "^[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}[-][0-9]{1,3}$";

char *reg_ip_addr =
    "^[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}\\.[0-9]{1,3}$";

char *reg_number =
    "\\d+?";
    
/*
 * Program usage
 */
void usage(FILE *out, char *error)
{
    char usage[] =
        "ReverseRaider domain scanner " VERSION " - " AUTHOR "\n" \
        "Usage:\n" \
        "  reverseraider -d domain | -r range [options]\n" \
        "Options:\n" \
        "  -d\tdomain, for wordlist scanning (example google.com)\n" \
        "  -r\trange, for reverse scanning (example 208.67.1.1-254)\n" \
        "  -w\twordlist file (see wordlists directory...)\n" \
        "  -t\tmax request time, in seconds\n" \
        "  -P\tenable numeric permutation on wordlist (default off)\n";

    fputs(usage, out);

    if(error)
        fputs(error, out);

    exit(1);
}

/*
 * Regular expression matching function
 * Returns:
 *    -1  error on pattern compilation
 *     0  don't match
 *     1  match
 */
int regmatch(const char* str, char* regex)
{
    regex_t re;

    // Compile the regex pattern.
    if (regcomp(&re, regex, REG_EXTENDED | REG_NOSUB) != 0) {
        fprintf(stderr, "Error on regex pattern compilation.\n");
        return -1;
    }

    // Use the pattern on the passed string
    if (regexec(&re, str, 0, NULL, 0))  {
        return 0;
    }

    regfree(&re);
    return 1;
}

/*
 * Parser for command line options
 * See getopt(3)...
 */
int parseopt(struct options *opt, int argc, char **argv)
{
    char *c=NULL;
    char ch;
    int i=0;
    
    char *domain=NULL;   // domain
    char range[MIN_ARG]; // range
    char *wordlist=NULL; // wordlist
    int time=0;          // time
    int permutation=0;   // permutation
    
    range[0]='\0';
    memset(opt, 0, sizeof(struct options)); // clear opt
         
    char *shortopt = "d:r:w:t:P"; // short options
    
    while ((ch = getopt (argc, argv, shortopt)) != -1)
        switch (ch) {
        
            case 'd': // Domain
                domain = optarg;
                break;
            
            case 'r': // Range
                if(!regmatch(optarg, reg_ip_range) && !regmatch(optarg, reg_ip_addr))
                    usage(stderr, "\nArgument range is invalid.\n");
                strncpy(range, optarg, MIN_ARG);
                break;
            
            case 'w': // Wordlist
                wordlist = optarg;
                break;
            
            case 't': // Time
                if(!regmatch(optarg, reg_number))
                    usage(stderr, "\nArgument time is invalid.\n");
                time = atoi(optarg);
                break;
            
            case 'P': // Permutation
                permutation = 1;
                break;
            
            case '?':
            default:
                usage(stderr, NULL);
        }

    //check collected args and fill options struct
    if(!domain && range[0]=='\0')
        usage(stderr, "\nDomain or range? Uff...\n");

    if(domain && !wordlist)
        usage(stderr, "\nYou forgot something (wordlist) :p\n");

    // domain
    if(domain) {
        strncpy(opt->domain, domain, MAX_ARG);
        strncpy(opt->wordlist, wordlist, MAX_ARG);
    }

    // time
    opt->time = time>0 ? time : 0;
    
    // permutation
    opt->permutation = permutation;
    
    // range
	if(range[0]!='\0') {
	
        strncpy(opt->range, range, MIN_ARG);

        // block
        while(range[i]!='\0') i++;
        while(range[i]!='.') i--;
        c=&range[i];

        *c = '\0';
        strncpy(opt->block, range, MIN_ARG);
        *c = '.';

        // start
        c = strtok(range, ".");
        for(i=0; i<3; i++) c=strtok(NULL, ".");

        opt->start=atoi(c);

        // end (single ip or range?)
        
        // - single ip
        if(regmatch(opt->range, reg_ip_addr)) {
            strcat(opt->range,"-");
            strcat(opt->range,c);
            opt->end=opt->start;
        }
        
        // - range
        else {
            i=0;
            while(range[i]!='-') i++;
            c = &range[i+1];
            
            opt->end=atoi(c);
        }
	}
	
    return 1;
}

#endif /* __REVERSERAIDER_H__ */

